@@ -25,7 +25,9 @@
 
 #include <unistd.h>
 #include <errno.h>
+#ifndef __MINGW32__
 #include <pwd.h>
+#endif
 #include <rfb/rfbclient.h>
 #ifdef LIBVNCSERVER_HAVE_LIBZ
 #include <zlib.h>
@@ -43,7 +45,7 @@
 rfbBool rfbEnableClientLogging=TRUE;
 
 void
-rfbClientLog(const char *format, ...)
+rfbDefaultClientLog(const char *format, ...)
 {
     va_list args;
     char buf[256];
@@ -64,6 +66,9 @@ rfbClientLog(const char *format, ...)
     va_end(args);
 }
 
+rfbClientLogProc rfbClientLog=rfbDefaultClientLog;
+rfbClientLogProc rfbClientErr=rfbDefaultClientLog;
+
 void FillRectangle(rfbClient* client, int x, int y, int w, int h, uint32_t colour) {
   int i,j;
 
@@ -175,8 +180,7 @@ static rfbBool decompStreamInited = FALSE;
  */
 
 /* Separate buffer for compressed data. */
-// TODO:
-// #define ZLIB_BUFFER_SIZE 512
+/* TODO: threading issues */
 #define ZLIB_BUFFER_SIZE 30000
 static char zlib_buffer[ZLIB_BUFFER_SIZE];
 
@@ -205,6 +209,33 @@ ConnectToRFBServer(rfbClient* client,const char *hostname, int port)
 {
   unsigned int host;
 
+  if (client->serverPort==-1) {
+    /* serverHost is a file recorded by vncrec. */
+    const char* magic="vncLog0.0";
+    char buffer[10];
+    rfbVNCRec* rec = (rfbVNCRec*)malloc(sizeof(rfbVNCRec));
+    client->vncRec = rec;
+
+    rec->file = fopen(client->serverHost,"rb");
+    rec->tv.tv_sec = 0;
+    rec->readTimestamp = FALSE;
+    rec->doNotSleep = FALSE;
+    
+    if (!rec->file) {
+      rfbClientLog("Could not open %s.\n",client->serverHost);
+      return FALSE;
+    }
+    setbuf(rec->file,0);
+    fread(buffer,1,strlen(magic),rec->file);
+    if (strncmp(buffer,magic,strlen(magic))) {
+      rfbClientLog("File %s was not recorded by vncrec.\n",client->serverHost);
+      fclose(rec->file);
+      return FALSE;
+    }
+    client->sock = 0;
+    return TRUE;
+  }
+
   if (!StringToIPAddr(hostname, &host)) {
     rfbClientLog("Couldn't convert '%s' to host address\n", hostname);
     return FALSE;
@@ -220,7 +251,7 @@ ConnectToRFBServer(rfbClient* client,const char *hostname, int port)
   return SetNonBlocking(client->sock);
 }
 
-static void rfbEncryptBytes(unsigned char *bytes, char *passwd);
+extern void rfbClientEncryptBytes(unsigned char* bytes, char* passwd);
 
 /*
  * InitialiseRFBConnection.
@@ -245,13 +276,14 @@ InitialiseRFBConnection(rfbClient* client)
     errorMessageOnReadFailure = FALSE;
 
   if (!ReadFromRFBServer(client, pv, sz_rfbProtocolVersionMsg)) return FALSE;
+  pv[sz_rfbProtocolVersionMsg]=0;
 
   errorMessageOnReadFailure = TRUE;
 
   pv[sz_rfbProtocolVersionMsg] = 0;
 
   if (sscanf(pv,rfbProtocolVersionFormat,&major,&minor) != 2) {
-    rfbClientLog("Not a valid VNC server\n");
+    rfbClientLog("Not a valid VNC server (%s)\n",pv);
     return FALSE;
   }
 
@@ -289,25 +321,28 @@ InitialiseRFBConnection(rfbClient* client)
   case rfbVncAuth:
     if (!ReadFromRFBServer(client, (char *)challenge, CHALLENGESIZE)) return FALSE;
 
-    if (client->GetPassword)
-      passwd = client->GetPassword(client);
+    if (client->serverPort!=-1) { /* if not playing a vncrec file */
+      if (client->GetPassword)
+        passwd = client->GetPassword(client);
 
-    if ((!passwd) || (strlen(passwd) == 0)) {
-      rfbClientLog("Reading password failed\n");
-      return FALSE;
+      if ((!passwd) || (strlen(passwd) == 0)) {
+        rfbClientLog("Reading password failed\n");
+        return FALSE;
+      }
+      if (strlen(passwd) > 8) {
+        passwd[8] = '\0';
+      }
+
+      rfbClientEncryptBytes(challenge, passwd);
+
+      /* Lose the password from memory */
+      for (i = strlen(passwd); i >= 0; i--) {
+        passwd[i] = '\0';
+      }
+      free(passwd);
+
+      if (!WriteToRFBServer(client, (char *)challenge, CHALLENGESIZE)) return FALSE;
     }
-    if (strlen(passwd) > 8) {
-      passwd[8] = '\0';
-    }
-
-    rfbEncryptBytes(challenge, passwd);
-
-	/* Lose the password from memory */
-    for (i = strlen(passwd); i >= 0; i--) {
-      passwd[i] = '\0';
-    }
-
-    if (!WriteToRFBServer(client, (char *)challenge, CHALLENGESIZE)) return FALSE;
 
     if (!ReadFromRFBServer(client, (char *)&authResult, 4)) return FALSE;
 
@@ -432,6 +467,12 @@ SetFormatAndEncodings(rfbClient* client)
 	encs[se->nEncodings++] = rfbClientSwap32IfLE(rfbEncodingZlib);
 	if (client->appData.compressLevel >= 0 && client->appData.compressLevel <= 9)
 	  requestCompressLevel = TRUE;
+      } else if (strncasecmp(encStr,"zlibhex",encStrLen) == 0) {
+	encs[se->nEncodings++] = rfbClientSwap32IfLE(rfbEncodingZlibHex);
+	if (client->appData.compressLevel >= 0 && client->appData.compressLevel <= 9)
+	  requestCompressLevel = TRUE;
+      } else if (strncasecmp(encStr,"zrle",encStrLen) == 0) {
+	encs[se->nEncodings++] = rfbClientSwap32IfLE(rfbEncodingZRLE);
 #endif
       } else if (strncasecmp(encStr,"corre",encStrLen) == 0) {
 	encs[se->nEncodings++] = rfbClientSwap32IfLE(rfbEncodingCoRRE);
@@ -632,6 +673,8 @@ HandleRFBServerMessage(rfbClient* client)
 {
   rfbServerToClientMsg msg;
 
+  if (client->serverPort==-1)
+    client->vncRec->readTimestamp = TRUE;
   if (!ReadFromRFBServer(client, (char *)&msg, 1))
     return FALSE;
 
@@ -728,26 +771,27 @@ HandleRFBServerMessage(rfbClient* client)
 
       switch (rect.encoding) {
 
-      case rfbEncodingRaw:
+      case rfbEncodingRaw: {
+	int y=rect.r.y, h=rect.r.h;
 
 	bytesPerLine = rect.r.w * client->format.bitsPerPixel / 8;
-	linesToRead = BUFFER_SIZE / bytesPerLine;
+	linesToRead = RFB_BUFFER_SIZE / bytesPerLine;
 
-	while (rect.r.h > 0) {
-	  if (linesToRead > rect.r.h)
-	    linesToRead = rect.r.h;
+	while (h > 0) {
+	  if (linesToRead > h)
+	    linesToRead = h;
 
 	  if (!ReadFromRFBServer(client, client->buffer,bytesPerLine * linesToRead))
 	    return FALSE;
 
 	  CopyRectangle(client, client->buffer,
-			   rect.r.x, rect.r.y, rect.r.w,linesToRead);
+			   rect.r.x, y, rect.r.w,linesToRead);
 
-	  rect.r.h -= linesToRead;
-	  rect.r.y += linesToRead;
+	  h -= linesToRead;
+	  y += linesToRead;
 
 	}
-	break;
+      } break;
 
       case rfbEncodingCopyRect:
       {
@@ -1098,14 +1142,14 @@ JpegSetSrcManager(j_decompress_ptr cinfo, uint8_t *compressedData,
 
 /* avoid name clashes with LibVNCServer */
 
-#define vncEncryptBytes rfbEncryptBytes
-#define vncEncryptAndStorePasswd rfbEncryptAndStorePasswdUnused
-#define vncDecryptPasswdFromFile rfbDecryptPasswdFromFileUnused
-#define vncRandomBytes rfbRandomBytesUnused
-#define des rfbDES
-#define deskey rfbDESKey
-#define usekey rfbUseKey
-#define cpkey rfbCPKey
+#define rfbEncryptBytes rfbClientEncryptBytes
+#define rfbEncryptAndStorePasswd rfbClientEncryptAndStorePasswdUnused
+#define rfbDecryptPasswdFromFile rfbClientDecryptPasswdFromFileUnused
+#define rfbRandomBytes rfbClientRandomBytesUnused
+#define rfbDes rfbClientDes
+#define rfbDesKey rfbClientDesKey
+#define rfbUseKey rfbClientUseKey
+#define rfbCPKey rfbClientCPKey
 
-#include "../vncauth.c"
-#include "../d3des.c"
+#include "../libvncserver/vncauth.c"
+#include "../libvncserver/d3des.c"
